//css_ref System.Data;
//css_ref System.Transactions;
//css_ref System.Xml;
//css_ref Interop.SAPbobsCOM.dll;
//css_ref log4net.dll;
//css_ref Produmex.Foundation.dll;
//using Produmex.Foundation.Data.dll;
//css_ref Produmex.Foundation.Data.Sbo.dll;
//using Produmex.Foundation.Sbo.dll;
//css_ref Produmex.Sbo.Logex.Data.dll;
//css_ref Produmex.Sbo.Logex.Shamrock.dll;

using System;
using System.Diagnostics;
using System.Reflection;
using System.Transactions;
using Produmex.Foundation.Data.Sbo;
using Produmex.Foundation.Data.Sbo.Attributes;
using Produmex.Foundation.Data.Sbo.BusinessObjects;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Convertors;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions.Tables;
using Produmex.Foundation.Data.Sbo.Providers;
using Produmex.Foundation.Data.Sbo.Utilities;
using Produmex.Foundation.Diagnostics;
using Produmex.Foundation.Reflection;
using Produmex.Foundation.Resources;
using Produmex.Sbo.Logex.Data.BusinessObjects;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions;
using Produmex.Sbo.Logex.Data.Providers;
using Produmex.Sbo.Logex.Shamrock.Extensions.Interfacing;
using Produmex.Sbo.Logex.Shamrock.Generators.ShamrockController;
using Produmex.Sbo.Logex.Shamrock.BusinessObjects.Definitions;

namespace AddUpdateScript
{
    class Program
    {
        static string CONNECTION_STRING = "Server=FERKOA-PC;CompanyDB=DEMO_PMX_debug;ExtraDb=PMX_DEMO_PMX_debug;UseTrusted=False;DbUserName=sa;DbPassword=sasa;UserName=manager;Password=produmex;Pooling=True;MinPoolSize=2;MaxPoolSize=10;ServerType=MSSQL2014;DBCultureInfo=en-US;LicenseServer=localhost:30000";
        
        /// <summary>
        /// Used for logging
        /// </summary>
        private static readonly ILog s_log = LogProvider.GetLogger(MethodInfo.GetCurrentMethod().DeclaringType);

        static void Main(string[] args)
        {
            try
            {
                Produmex.Foundation.Diagnostics.LogProvider.SetConsoleAppender("WARN");
                Execute();
            }
            catch (Exception ex)
            {
                s_log.Error("", ex);
            }
            if (Debugger.IsAttached)
            {
                Console.WriteLine("Press <Enter>");
                Console.ReadLine();
            }
        }

        static void Execute()
        {
            using (TransactionScope scope = PmxDbConnection.GetNewTransactionScope())
            {
                using (SboConnection conn = new SboConnection(CONNECTION_STRING))
                {
                    conn.Open(); //Open connection to SAP
                    Console.WriteLine("Connection is open to database");

                    // PmxDbVersionUpdaterProvider updateProv = new PmxDbVersionUpdaterProvider(conn);

                    // * UDF's *

                    //Console.WriteLine("Extensions");


                }

                using (PmxDbConnectionSql dbConn = new PmxDbConnectionSql(CONNECTION_STRING))
                {
                    dbConn.Open(); //Open connection to SAP
                    Console.WriteLine("Connection is open to database");

                    //Console.WriteLine("Workflows");

                    Console.WriteLine("Extensions");

                    AddUpdateExtension(
                         PmxAdvanceShippingNoticeInterfaceShamrock.IDs.ExtCode,
                         "IPmxAdvanceShippingNoticeInterface - Shamrock",
                         typeof(PmxAdvanceShippingNoticeInterfaceShamrock),
                         Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.PmxExtensionTypeCodes.IADVANCESHIPPINGNOTICE_INTERFACE,
                         dbConn
                         );

                    //parameters
                    AddUpdateExtensionParameter(
                        ShamrockController.IDs.EXTPARAMBATCHATTRIBUTEMAPPINGCODE,
                        "Batch attribute mapping code",
                        ShamrockController.IDs.ExtCode,
                        Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions.PmxExtensionCodes.SYSTEM_STRING_CONVERTOR,
                        PmxShamrockExtensionParameterCodes.BATCHATTRIBUTEMAPPINGCODE_DEFVAL,
                        PmxShamrockExtensionParameterCodes.BATCHATTRIBUTEMAPPINGCODE, 0, dbConn);
                }

                //Complete transaction
                scope.Complete();
            }
            
            Console.WriteLine("Done ...");
        }


        /// <summary>
        /// Add/Update <see cref="PmxExtension"/>
        /// </summary>
        /// <param name="code">The code of the extension.</param>
        /// <param name="name">The name of the extension.</param>
        /// <param name="type">The type of the extension.</param>
        /// <param name="extensionTypeCode">The extension type code.</param>
        /// <param name="conn">The SBO connection</param>
        private static void AddUpdateExtension(string code, string name, Type type, string extensionTypeCode, PmxDbConnectionSql conn)
        {
            PmxExtensionProvider prov = new PmxExtensionProvider(conn);
            PmxExtension extension = null;
            try
            {
                bool newBO = false;
                extension = prov.GetBO(code);
                if (extension == null)
                {
                    newBO = true;
                    extension = prov.GetNewBO();
                    extension.Code = code;
                }
                extension.Name = name;
                extension.ExtensionTypeCode = extensionTypeCode;
                extension.Type = ReflectionHelper.GetTypeAndAssemblyName(type);
                if (newBO)
                {
                    prov.AddBO(extension);
                }
                else
                {
                    prov.UpdateBO(extension, false, false);
                }
            }
            finally
            {
                if (extension != null) extension.Dispose();
            }
        }

        /// <summary>
        /// Add/Update <see cref="PmxExtension"/>
        /// </summary>
        /// <param name="code">The code of the extension.</param>
        /// <param name="name">The name of the extension.</param>
        /// <param name="type">The type of the extension.</param>
        /// <param name="extensionTypeCode">The extension type code.</param>
        /// <param name="conn">The DB connection</param>
        private static void AddUpdateExtension(string code, string name, Type type, string extensionTypeCode, PmxDbConnection conn)
        {
            PmxExtensionProvider prov = new PmxExtensionProvider(conn);
            PmxExtension extension = null;
            try
            {
                bool newBO = false;
                extension = prov.GetBO(code);
                if (extension == null)
                {
                    newBO = true;
                    extension = prov.GetNewBO();
                    extension.Code = code;
                }
                extension.Name = name;
                extension.ExtensionTypeCode = extensionTypeCode;
                extension.Type = ReflectionHelper.GetTypeAndAssemblyName(type);
                if (newBO)
                {
                    prov.AddBO(extension);
                }
                else
                {
                    prov.UpdateBO(extension, false, false);
                }
            }
            finally
            {
                if (extension != null)
                    extension.Dispose();
            }
        }

        /// <summary>
        /// Add/update <see cref="PmxExtensionType"/>
        /// </summary>
        /// <param name="code">The code of the extension type.</param>
        /// <param name="name">The name of the extension type.</param>
        /// <param name="interfaceType">The interface type of the extension type.</param>
        /// <param name="conn">The DB connection</param>
        public static void AddUpdateExtensionType(string code, string name, Type interfaceType, PmxDbConnection conn)
        {
            PmxExtensionTypeProvider prov = new PmxExtensionTypeProvider(conn);
            PmxExtensionType exType = null;
            try
            {
                bool newBO = false;
                exType = prov.GetBO(code);
                if (exType == null)
                {
                    newBO = true;
                    exType = prov.GetNewBO();
                    exType.Code = code;
                }
                exType.Name = name;
                exType.InterfaceType = ReflectionHelper.GetTypeAndAssemblyName(interfaceType);
                if (newBO)
                {
                    prov.AddBO(exType);
                }
                else
                {
                    prov.UpdateBO(exType, false, false);
                }
            }
            finally
            {
                if (exType != null)
                    exType.Dispose();
            }
        }

        /// <summary>
        /// Add/Update <see cref="PmxExtensionParameter"/>
        /// </summary>
        /// <param name="code">The code of the extension parameter.</param>
        /// <param name="name">The name of the extension parameter.</param>
        /// <param name="extensionCode">The extension code.</param>
        /// <param name="convertorCode">The convertor code.</param>
        /// <param name="defaultValue">The default value.</param>
        /// <param name="generalParameterCode">The general parameter code.</param>
        /// <param name="sequence">The sequence.</param>
        /// <param name="conn">The DB connection</param>
        public static void AddUpdateExtensionParameter(string code, string name, string extensionCode, string convertorCode, string defaultValue, string generalParameterCode, int sequence, PmxDbConnection conn)
        {
            PmxExtensionParameterProvider prov = new PmxExtensionParameterProvider(conn);
            PmxExtensionParameter extParam = null;
            try
            {
                bool newBO = false;
                extParam = prov.GetBO(code);
                if (extParam == null)
                {
                    newBO = true;
                    extParam = prov.GetNewBO();
                    extParam.Code = code;
                }
                extParam.Name = name;
                extParam.ExtensionCode = extensionCode;
                extParam.ConvertorCode = convertorCode;
                extParam.GeneralParameterCode = generalParameterCode;
                extParam.DefaultValue = defaultValue;
                extParam.Sequence = sequence;
                if (newBO)
                {
                    prov.AddBO(extParam);
                }
                else
                {
                    prov.UpdateBO(extParam, false, false);
                }
            }
            finally
            {
                if (extParam != null)
                    extParam.Dispose();
            }
        }

        /// <summary>
        /// Add/Update <see cref="PmxProperty"/>
        /// </summary>
        /// <param name="code">The code of the property.</param>
        /// <param name="name">The name of the property.</param>
        /// <param name="extensionTypeCode">The extension type code.</param>
        /// <param name="defaultNewExtensionCode">The default new extension code.</param>
        /// <param name="conn">The DB connection</param>
        public static void AddUpdateProperty(string code, string name, string extensionTypeCode, string defaultNewExtensionCode, PmxDbConnection conn)
        {
            PmxPropertyProvider prov = new PmxPropertyProvider(conn);
            PmxProperty property = null;
            try
            {
                bool newBO = false;
                property = prov.GetBO(code);
                if (property == null)
                {
                    newBO = true;
                    property = prov.GetNewBO();
                    property.Code = code;
                }
                property.Name = name;
                property.ExtensionTypeCode = extensionTypeCode;

                if (newBO)
                {
                    if (!string.IsNullOrEmpty(defaultNewExtensionCode))
                    {
                        //only set when defaulExtansionCode is set, and the property is new
                        property.ExtensionCode = defaultNewExtensionCode;
                    }
                    prov.AddBO(property);
                }
                else
                {
                    prov.UpdateBO(property, false, false);
                }
            }
            finally
            {
                if (property != null)
                    property.Dispose();
            }
        }



        /// <summary>
        /// Add/Update script workflow
        /// </summary>
        /// <param name="executionType">Type of the execution.</param>
        /// <param name="name">The name of the flow.</param>
        /// <param name="sourceFilePath">Name of the file.</param>
        /// <param name="sqlConn">The SQL conn.</param>
        private static void AddUpdateWorkflowScript(string executionType, string name, string sourceFilePath, PmxDbConnectionSql sqlConn)
        {
            PmxWorkflowProvider wfProvider = new PmxWorkflowProvider(sqlConn);
            PmxWorkflow workflow = wfProvider.GetWorkflowByName(name);
            PmxWorkflowScriptProvider wfscProvider = new PmxWorkflowScriptProvider(sqlConn);
            PmxWorkflowScript bo = null;
            if (workflow == null)
            {
                //add the workflow
                bo = wfscProvider.GetNewBO();
                bo.Name = name;
                wfscProvider.AddBO(bo);
            }
            else
            {
                if (workflow.ObjType == Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.PmxObjectTypes.PmxWorkflowScript)
                {
                    bo = wfscProvider.GetBO(workflow.InternalKey);

                }
            }

            if (bo != null)
            {
                bo.ExecutionType = executionType;

                wfscProvider.UpdateSourceFromFileAndCompile(bo, sourceFilePath, sqlConn);
            }



        }

        /// <summary>
        /// Runs the table SQL script.
        /// </summary>
        /// <param name="tableScript">The table script.</param>
        /// <param name="conn">The conn.</param>
        private static void RunTableSqlScript(string tableScript, PmxDbConnection conn)
        {
            string content = ResourceHelper.ReadResourceContentAsString(Assembly.GetAssembly(typeof(Produmex.Sbo.Logex.Shamrock.SboCommands)), "Produmex.Sbo.Logex.Shamrock.Scripts.Tables." + tableScript);
            SboRecordsetHelper.RunSqlScript(s_log, content, conn);
        }

        /// <summary>
        /// Adds the update user fields not in transaction.
        /// </summary>
        /// <param name="conn">The conn.</param>
        /// <param name="types">The types.</param>
        private static void AddUpdateUserFieldsNotInTransaction(SboConnection conn, params Type[] types)
        {
            SboUserFieldMDProvider userFieldMDProv = new SboUserFieldMDProvider(conn);
            //System.Data.Common.DbTransaction transaction = null;
            try
            {
                //transaction = this.Connection.BeginTransaction();
                userFieldMDProv.AddUpdateUserFields(types);

                //Commit transaction
                //transaction.Commit();
            }
            catch (Exception)
            {
                //if (transaction != null) transaction.Rollback();
                throw;
            }
        }

        /// <summary>
        /// Runs the stored procedure SQL script.
        /// </summary>
        /// <param name="spScript">The sp script.</param>
        /// <param name="conn">The conn.</param>
        private static void RunStoredProcedureSqlScript(string spScript, PmxDbConnection conn)
        {
            string content = ResourceHelper.ReadResourceContentAsString(Assembly.GetAssembly(typeof(Produmex.Sbo.Logex.Shamrock.SboCommands)), "Produmex.Sbo.Logex.Shamrock.Scripts.StoredProcedures." + spScript);
            SboRecordsetHelper.RunSqlScript(s_log, content, conn);
        }


    }
}
